from django.utils.translation import gettext_noop
from django.http import HttpResponseRedirect
from django.contrib.auth import REDIRECT_FIELD_NAME, authenticate, login
from django.shortcuts import render
from django.conf import settings

from authentic2.constants import NONCE_FIELD_NAME
from authentic2.models import AuthenticationEvent

from . import constants, forms, models

class PraticFrontend(object):
    def enabled(self):
        True

    def id(self):
        return 'pratic'

    def name(self):
        return gettext_noop('Pr@tic')

    def login(self, request, *args, **kwargs):
        next_url = request.GET.get(REDIRECT_FIELD_NAME) or settings.LOGIN_REDIRECT_URL
        nonce = request.GET.get(NONCE_FIELD_NAME,'')
        # if there is only one ssl user, autolog him
        # only autologin every 10 minutes
        user = None
        how = None
        autologin = False
        if constants.PRATIC_AUTOLOGIN_COOKIE_NAME not in request.COOKIES:
            if hasattr(request, 'ssl_info') and request.ssl_info and \
                len(request.ssl_info.users) == 1:
                    user = authenticate(ssl_user=request.ssl_info.users[0])
                    how = 'ssl'
                    autologin = True
        # login SSL users by selection
        if not user and request.method == 'POST' and 'ssl-user' in request.POST \
                and hasattr(request, 'ssl_info') and request.ssl_info:
            try:
                ssl_user_id = int(request.POST['ssl-user'])
            except ValueError:
                pass
            else:
                try:
                    ssl_user = request.ssl_info.users.get(pk=ssl_user_id)
                except models.User.DoesNotExist:
                    pass
                else:
                    user = authenticate(ssl_user=ssl_user)
                    how = 'ssl'
        # now try to login using collectivity/login/password
        form = forms.AuthenticationForm(request=request, data=request.POST or None)
        if not user and request.method == 'POST' and 'pratic-login' in request.POST \
                and form.is_valid():
            how = form.get_how()
            if how == 'password' and request.is_secure():
                how += '-on-https'
            user = form.get_user()
        if user and how:
            login(request, user)
            AuthenticationEvent.objects.create(who=user.username,
                    how=how, nonce=nonce)
            response = HttpResponseRedirect(next_url)
            if autologin:
                response.set_cookie(constants.PRATIC_AUTOLOGIN_COOKIE_NAME,
                        max_age=constants.PRATIC_AUTOLOGIN_COOKIE_MAX_AGE)
            return response
        return render(request, 'authentic2_pratic/login.html',
                {'form': form})
